package eu.DECIDEh2020.mcsla.service.controller;

/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* <<licensing_schema_to_be_decided>> which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Majid Salehi Ghamsari Fraunhofer FOKUS
**Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
*/
import java.util.List;

import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import eu.DECIDEh2020.mcsla.beans.Csp;
import eu.DECIDEh2020.mcsla.beans.Mcsla;
import eu.DECIDEh2020.mcsla.beans.MicroserviceSLA;
import eu.DECIDEh2020.mcsla.beans.MicroserviceSLO;
import eu.DECIDEh2020.mcsla.service.configuration.ProjectConfig;
import eu.DECIDEh2020.mcsla.service.service.McslaService;

@RestController
public class McslaController {

	McslaService mcslaService = new McslaService();

	// operation on MCSLA repo
	@RequestMapping(value = "/mcsla", method = RequestMethod.GET, headers = "Accept=application/json")
	public Mcsla getMcslaFromRepo() {
		System.out.println("McslaController:getMcslaFromRepo:----------------------");
		return mcslaService.getMcsla();
	}

	// operation on MCSLAs
	@RequestMapping(value = "/mcslas", method = RequestMethod.GET, headers = "Accept=application/json")
	public List<Mcsla> getMcslas() {
		System.out.println("McslaController:getMcslas:----------------------");
		List<Mcsla> listOfMcslas = mcslaService.getAllMcslas();
		return listOfMcslas;
	}

	@RequestMapping(value = "/mcslas/{id}", method = RequestMethod.GET, headers = "Accept=application/json")
	public Mcsla getMcslaById(@PathVariable String id) {
		System.out.println("McslaController:getMcslaById:----------------------");
		return mcslaService.getMcsla(id);
	}

	// add MCSLA to repository and update the reference in Application description
	@RequestMapping(value = "/mcslas", method = RequestMethod.POST, headers = "Accept=application/json")
	public Mcsla addMcsla(@RequestBody Mcsla mcsla) {
		System.out.println("McslaController:addMcsla:----------------------");
		return mcslaService.addMcsla(mcsla);
	}

	@RequestMapping(value = "/mcslas", method = RequestMethod.PUT, headers = "Accept=application/json")
	public Mcsla updateMcsla(@RequestBody Mcsla mcsla) {
		System.out.println("McslaController:updateMcsla:----------------------");
		return mcslaService.updateMcsla(mcsla);
	}

	@RequestMapping(value = "/mcslas/{id}", method = RequestMethod.DELETE, headers = "Accept=application/json")
	public void deleteMcsla(@PathVariable("id") String id) {
		System.out.println("McslaController:deleteMcsla:----------------------");
		mcslaService.deleteMcsla(id);
	}

	// operation on SLOs
	@RequestMapping(value = "/aggregate", method = RequestMethod.POST, headers = "Accept=application/json")
	public MicroserviceSLO aggregate(@RequestBody List<MicroserviceSLO> slos) {
		System.out.println("McslaController:aggregate:----------------------");
		return mcslaService.aggregate(slos);
	}

	@RequestMapping(value = "/config", method = RequestMethod.GET, headers = "Accept=application/json")
	public ProjectConfig getProjectConfig() {
		System.out.println("McslaController:getProjectConfig:----------------------");
		ProjectConfig projectConfig = mcslaService.getProjectConfig();
		return projectConfig;
	}

	// operation on CSPs will be delivered later by acsmi
	@RequestMapping(value = "/csps", method = RequestMethod.GET, headers = "Accept=application/json")
	public List<Csp> getCsps() {
		System.out.println("McslaController:getCsps:----------------------");
		List<Csp> listOfCsps = mcslaService.getAllCsps();
		return listOfCsps;
	}

	// operation on CSPs will be delivered later by acsmi
	@RequestMapping(value = "/csps/{id}", method = RequestMethod.GET, headers = "Accept=application/json")
	public List<MicroserviceSLA> getCspSLAsById(@PathVariable String id) {
		System.out.println("McslaController:getCspSLAsById:----------------------");
		return mcslaService.getCspSLAs(id);
	}
}
